@@ -3,9 +3,7 @@
 Package containing implementation of all the standard Distutils
 commands."""
 
-# This module should be kept compatible with Python 1.5.2.
-
-__revision__ = "$Id: __init__.py 31709 2003-03-03 18:37:16Z akuchling $"
+__revision__ = "$Id: __init__.py 71473 2009-04-11 14:55:07Z tarek.ziade $"
 
 __all__ = ['build',
            'build_py',
@@ -24,6 +22,8 @@ __all__ = ['build',
            'bdist_dumb',
            'bdist_rpm',
            'bdist_wininst',
+           'upload',
+           'check',
            # These two are reserved for future use:
            #'bdist_sdux',
            #'bdist_pkgtool',
